#!/usr/bin/env python
from __future__ import absolute_import, print_function

# #############################################################################
# Copyright : (C) 2017 by Teledomic.eu All rights reserved
#
# Name:         mytb.tests.test_argparse
#
# Description:  unit tests for argparser
#
# #############################################################################

import unittest


class ArgParseTestCase(unittest.TestCase):
    def test_mkparser(self):
        """ 1 1 2 """
        import mytb.argparse
        parser = mytb.argparse.mk_parser()

        #  find all expected keys in parser
        options = parser.parse_args([])
        self.assertEqual(
            set(vars(options).keys()),
            set(['app_name', 'log_config']))
        self.assertEqual(options.log_config, None)

        #  -L switch works
        options = parser.parse_args(['-L', 'tido'])
        self.assertEqual(options.log_config, 'tido')

        #  --log-config switch works
        options = parser.parse_args(['--log-config', 'todo'])
        self.assertEqual(options.log_config, 'todo')

        #  can override long switch
        mytb.argparse.LONG_LOG_SWITCH = '--lock-cfg'
        parser = mytb.argparse.mk_parser()
        options = parser.parse_args(['--lock-cfg', 'tata'])
        self.assertEqual(options.lock_cfg, 'tata')
        print(options)



from __future__ import absolute_import
from __future__ import print_function


# -----------------------------------------------------------------------------
#   Imports
# -----------------------------------------------------------------------------
import threading
import logging

from builtins import input


logger = logging.getLogger(__name__)


class CLI(object):
    input_func = input

    def __init__(self, options=None, namespace=None, quit_func=None):
        self._cli_thread = None
        self._options = options
        if namespace is None:
            namespace = dict(__lock=threading.Lock())
        self.namespace = namespace
        if not hasattr(namespace, '__lock'):
            namespace.update(dict(__lock=threading.Lock()))
        self._lock = namespace['__lock']
        self._quit_function = quit_func

    def set_quit_function(self, func):
        self._quit_function = func

    def run(self):
        """ allows to run an ipython shell with the CLI's context vars """
        namespace = self.namespace
        try:
            from IPython.terminal.embed import InteractiveShellEmbed
            use_ipython = True
            logger.debug("CLI using ipython")
        except ImportError:
            use_ipython = False
            logger.debug("CLI using basic fallback")

        if use_ipython:
            shell = InteractiveShellEmbed(user_ns=namespace)
            shell()

        else:
            self.mini_shell(namespace=namespace)

        if self._quit_function:
            try:
                self._quit_function(self)
            except TypeError:
                logger.warning("using obsolete quit function without argument")
                self._quit_function()

    def mini_shell(self, namespace):
        """ Rather lousy Python shell for debugging.
            Just in case ipython is not installed or has the wrong version
        """
        while True:
            cmd_line = self.input_func('-->')
            upper_stripped = cmd_line.strip().upper()
            shall_quit = (upper_stripped == 'Q' or upper_stripped == 'QUIT')
            if shall_quit:
                break
            try:
                eval(compile(cmd_line, '<string>', 'single'), namespace)
            except Exception as exc:
                logger.error('ERROR: %r' % exc)

        print("END OF CLI")
        self.write_history()

    def write_history(self, fname=None):
        pass

    def run_as_thread(self, name='cli', daemon=True):
        """ start cli as a thread
            This is needed for Qt Apps, where the GUI must be called
            in the main thread
        """
        self._cli_thread = cli_thread = threading.Thread(
            target=self.run,
            name=name)
        cli_thread.daemon = daemon
        cli_thread.start()


def main():
    pass


if __name__ == '__main__':
    main()
# -----------------------------------------------------------------------------
#   End of file
# -----------------------------------------------------------------------------



#!/usr/bin/env python

from __future__ import print_function
from __future__ import absolute_import

import sys
import csv

# 2/3 compatibility w future
from builtins import next
from builtins import object
from builtins import open


IS_PY2 = sys.version_info.major <= 2


class CSVReader(object):
    def __init__(self, fname, encoding='utf8'):
        self.fname = fname
        self.encoding = encoding
        self.fin = None
        self.rdr = None
        self.itr = iter(self)

    def __enter__(self):

        if IS_PY2:
            self.fin = open(self.fname, 'rb')
        else:
            self.fin = open(self.fname, encoding=self.encoding)
        self.rdr = csv.reader(self.fin)
        return self

    def __exit__(self, typ, value, traceback):
        self.fin.close()

    def __iter__(self):
        encoding = self.encoding
        if IS_PY2:
            for row in self.rdr:
                row = [field.decode(encoding)
                       if type(field) == bytes else field
                       for field in row]
                yield row
        else:
            for row in self.rdr:
                yield row

    def __next__(self):
        return next(self.itr)



#!/usr/bin/env python
from __future__ import absolute_import, print_function

# #############################################################################
# Copyright : (C) 2017 by Teledomic.eu All rights reserved
#
# Name:         mytb.importlib
#
# Description:  helper for locating / importing modules
#
# #############################################################################


def module_exists(modulename):
    """ tries to find out whether a module exists """
    import pkgutil
    moduleparts = modulename.split('.')
    mod = ""
    for part in moduleparts:
        mod += '.' + part if mod else part
        if pkgutil.find_loader(mod) is None:
            return False
    return True


def import_obj(obj_str):
    """ handles input string as module.object
        and imports the module and returns the object
    """
    import importlib
    if ':' in obj_str:
        modname, objname = obj_str.split(':')
    else:
        modname, objname = obj_str.rsplit('.', 1)
    mod = importlib.import_module(modname)
    func = getattr(mod, objname)
    return func



from __future__ import absolute_import, print_function

# python std modules

# third party modules
from setuptools import setup

install_requires = [
    'future',
    'begins',
    'dateutils',
    'minibelt',
    'pyyaml',
    'pytz',
    'tzlocal',
]

setup(name='mytb',
      version='0.0.5',
      description='my toolbox for everyday python projects',
      long_description="small modules and tools useful for many projects",
      long_description_content_type="text/x-rst",
      classifiers=[
            'Development Status :: 3 - Alpha',
      ],
      keywords='toolbox development',
      url='https://www.teledomic.eu',
      author='Teledomic',
      author_email='info@teledomic.eu',
      license="MIT",
      packages=[
            'mytb',
            'mytb.commands',
            'mytb.importlib',
            'mytb.logging',
            'mytb.gitlab',
            'mytb.logging.handlers',

            # might uncomment if you want that the dist package implements
            # no tests
            'mytb.tests',
            'mytb.tests.gitlab',
            ],
      scripts=[],
      entry_points={
          'console_scripts': [
            'mytb = mytb.commands:main',
            ]
      },
      project_urls={
        'Documentation': "https://github.com/feenes/mytb",
        'Source': "https://github.com/feenes/mytb",
        'SayThanks': "https://github.com/feenes",
        'Funding': 'https://donate.pypi.org',
        'Tracker': "https://github.com/feenes/mytb/issues",
      },
      test_suite='nose.collector',
      install_requires=install_requires,
      tests_require=['nose', 'ddt'],
      zip_safe=False,
      python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, <4',
      )
